<?php

namespace Hospital\Doctor\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hospital\Doctor\Http\Requests\DoctorRequest;
use Hospital\Doctor\Interfaces\DoctorRepositoryInterface;
use Hospital\Doctor\Models\Doctor;

/**
 * Resource controller class for doctor.
 */
class DoctorResourceController extends BaseController
{

    /**
     * Initialize doctor resource controller.
     *
     * @param type DoctorRepositoryInterface $doctor
     *
     * @return null
     */
    public function __construct(DoctorRepositoryInterface $doctor)
    {
        parent::__construct();
        $this->repository = $doctor;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hospital\Doctor\Repositories\Criteria\DoctorResourceCriteria::class);
    }

    /**
     * Display a list of doctor.
     *
     * @return Response
     */
    public function index(DoctorRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hospital\Doctor\Repositories\Presenter\DoctorPresenter::class)
                ->$function();
        }

        $doctors = $this->repository->paginate();

        return $this->response->title(trans('doctor::doctor.names'))
            ->view('doctor::doctor.index', true)
            ->data(compact('doctors'))
            ->output();
    }

    /**
     * Display doctor.
     *
     * @param Request $request
     * @param Model   $doctor
     *
     * @return Response
     */
    public function show(DoctorRequest $request, Doctor $doctor)
    {

        if ($doctor->exists) {
            $view = 'doctor::doctor.show';
        } else {
            $view = 'doctor::doctor.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('doctor::doctor.name'))
            ->data(compact('doctor'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new doctor.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DoctorRequest $request)
    {

        $doctor = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('doctor::doctor.name')) 
            ->view('doctor::doctor.create', true) 
            ->data(compact('doctor'))
            ->output();
    }

    /**
     * Create new doctor.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DoctorRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $doctor                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('doctor::doctor.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('doctor/doctor/' . $doctor->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/doctor/doctor'))
                ->redirect();
        }

    }

    /**
     * Show doctor for editing.
     *
     * @param Request $request
     * @param Model   $doctor
     *
     * @return Response
     */
    public function edit(DoctorRequest $request, Doctor $doctor)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('doctor::doctor.name'))
            ->view('doctor::doctor.edit', true)
            ->data(compact('doctor'))
            ->output();
    }

    /**
     * Update the doctor.
     *
     * @param Request $request
     * @param Model   $doctor
     *
     * @return Response
     */
    public function update(DoctorRequest $request, Doctor $doctor)
    {
        try {
            $attributes = $request->all();

            $doctor->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('doctor::doctor.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('doctor/doctor/' . $doctor->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('doctor/doctor/' . $doctor->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the doctor.
     *
     * @param Model   $doctor
     *
     * @return Response
     */
    public function destroy(DoctorRequest $request, Doctor $doctor)
    {
        try {

            $doctor->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('doctor::doctor.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('doctor/doctor/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('doctor/doctor/' . $doctor->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple doctor.
     *
     * @param Model   $doctor
     *
     * @return Response
     */
    public function delete(DoctorRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('doctor::doctor.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('doctor/doctor'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/doctor/doctor'))
                ->redirect();
        }

    }

    /**
     * Restore deleted doctors.
     *
     * @param Model   $doctor
     *
     * @return Response
     */
    public function restore(DoctorRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('doctor::doctor.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/doctor/doctor'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/doctor/doctor/'))
                ->redirect();
        }

    }

}
